@@ -4,7 +4,7 @@ use 5.010;
 use strict;
 use warnings;
 use base 'Net::Server::PreFork';
-our $VERSION = '2.20200107'; # VERSION
+our $VERSION = '2.20200202'; # VERSION
 
 
 use English qw{ -no_match_vars };
@@ -254,6 +254,7 @@ sub child_init_hook {
     $self->{'object_maker'}   = $object_maker;
 
     $self->setup_handlers();
+    $self->{'metric'}->set_versions( $self );
 
     $PROGRAM_NAME = $Mail::Milter::Authentication::Config::IDENT . ':waiting(0)';
     return;
@@ -861,7 +862,7 @@ sub setup_handler {
     my $object = $package->new( $self );
     $self->{'handler'}->{$name} = $object;
 
-    foreach my $callback ( qw { setup connect helo envfrom envrcpt header eoh body eom addheader abort close } ) {
+    foreach my $callback ( qw { metrics setup connect helo envfrom envrcpt header eoh body eom addheader abort close } ) {
         if ( $object->can( $callback . '_callback' ) ) {
             $self->register_callback( $name, $callback );
         }
@@ -895,7 +896,7 @@ sub register_callback {
 
 sub sort_all_callbacks {
     my ($self) = @_;
-    foreach my $callback ( qw { setup connect helo envfrom envrcpt header eoh body eom addheader abort close } ) {
+    foreach my $callback ( qw { metrics setup connect helo envfrom envrcpt header eoh body eom addheader abort close } ) {
         $self->sort_callbacks( $callback );
     }
     return;
@@ -1093,7 +1094,7 @@ Mail::Milter::Authentication - A Perl Mail Authentication Milter
 
 =head1 VERSION
 
-version 2.20200107
+version 2.20200202
 
 =head1 SYNOPSIS
 
