@@ -1,7 +1,7 @@
 package Mail::Milter::Authentication::Handler;
 use strict;
 use warnings;
-our $VERSION = '2.20200107'; # VERSION
+our $VERSION = '2.20200202'; # VERSION
 
 
 use Digest::MD5 qw{ md5_hex };
@@ -84,6 +84,22 @@ sub metric_count {
 }
 
 
+sub metric_set {
+    my ( $self, $gauge_id, $labels, $value ) = @_;
+    $labels = {} if ! defined $labels;
+    die 'Must set value in metric_set call' if ! defined $value;
+
+    my $metric = $self->{'thischild'}->{'metric'};
+    $metric->set({
+        'gauge_id' => $gauge_id,
+        'labels'   => $labels,
+        'server'   => $self->{'thischild'},
+        'value'    => $value,
+    });
+    return;
+}
+
+
 sub metric_send {
     my ( $self ) = @_;
     # NOOP
@@ -381,6 +397,21 @@ sub remap_connect_callback {
 }
 
 
+sub top_metrics_callback {
+    my ( $self ) = @_;
+    my $callbacks = $self->get_callbacks( 'metrics' );
+    foreach my $handler ( @$callbacks ) {
+        $self->dbgout( 'CALLBACK', 'Metrics ' . $handler, LOG_DEBUG );
+        eval{ $self->get_handler($handler)->metrics_callback(); };
+        if ( my $error = $@ ) {
+            $self->handle_exception( $error );
+            $self->log_error( 'Metrics callback error ' . $error );
+        }
+    };
+    return;
+}
+
+
 sub top_connect_callback {
 
     # On Connect
@@ -2071,7 +2102,7 @@ Mail::Milter::Authentication::Handler
 
 =head1 VERSION
 
-version 2.20200107
+version 2.20200202
 
 =head1 DESCRIPTION
 
@@ -2269,6 +2300,10 @@ Register a metric type
 
 Increment a metrics counter by $count (defaults to 1 if undef)
 
+=head2 I<metric_set( $id, $labels, $count )>
+
+Set a metrics counter to $count
+
 =head2 I<metric_send()>
 
 Send metrics to the parent
@@ -2364,6 +2399,10 @@ Top level handler for handler setup.
 
 Top level handler for the connect event for remapping only.
 
+=head2 I<top_metrics_callback()>
+
+Top level handler for the metrics event.
+
 =head2 I<top_connect_callback( $hostname, $ip )>
 
 Top level handler for the connect event.
