@@ -2,7 +2,7 @@ package Mail::Milter::Authentication::Handler::DMARC;
 use strict;
 use warnings;
 use base 'Mail::Milter::Authentication::Handler';
-our $VERSION = '2.20200107'; # VERSION
+our $VERSION = '2.20200202'; # VERSION
 
 use Data::Dumper;
 use English qw{ -no_match_vars };
@@ -179,9 +179,27 @@ sub pre_fork_setup {
 sub register_metrics {
     return {
         'dmarc_total' => 'The number of emails processed for DMARC',
+        'dmarc_reports_total' => { type => 'gauge', help => 'The number of pending DMARC reports' },
     };
 }
 
+sub metrics_callback {
+    my ( $self ) = @_;
+    my $config = $self->handler_config();
+    return if $config->{'no_report'};
+
+    eval {
+        my $time = time;
+        my $backend = Mail::DMARC::Report::Store->new()->backend;
+        my $current = $backend->query("SELECT COUNT(1) AS c FROM report WHERE end >= $time")->[0]->{c};
+        my $pending = $backend->query("SELECT COUNT(1) AS c FROM report WHERE end < $time")->[0]->{c};
+        $self->metric_set( 'dmarc_reports_total', { 'state' => 'current' }, $current );
+        $self->metric_set( 'dmarc_reports_total', { 'state' => 'pending' }, $pending );
+    };
+
+    return;
+}
+
 sub _process_arc_dmarc_for {
     my ( $self, $env_domain_from, $header_domain ) = @_;
 
@@ -434,10 +452,40 @@ sub _process_dmarc_for {
     if ( $dmarc_code eq 'fail' ) {
         # Policy override decisions.
         if ( $arc_aware_result eq 'pass' ) {
-            $self->dbgout( 'DMARCReject', "Policy overridden using ARC Chain", LOG_INFO );
             $dmarc_result->disposition('none');
             $dmarc_disposition = 'none';
-            $dmarc_result->reason( 'type' => 'trusted_forwarder', 'comment' => 'Policy overriden using trusted ARC chain' );
+            my $comment = 'Policy overriden using trusted ARC chain';
+            # arc=pass as[2].d=d2.example as[2].s=s2 as[1].d=d1.example as[1].s=s3 remote-ip[1]=2001:DB8::1A
+            my $arc_object = $self->get_object('arc');
+            my $arc_signatures = $arc_object->{'signatures'};
+
+            my $arc_handler = $self->get_handler('ARC');
+            if ( $arc_handler ) {
+              if ( $arc_handler->{ 'arc_result' } eq 'pass' ) {
+                # If it wasn't a pass then we wouldn't be in here.
+                $comment = 'arc=pass';
+                my $arc_auth_results = $arc_handler->{'arc_auth_results'};
+                foreach my $instance ( reverse sort keys %$arc_auth_results ) {
+                  my $domain = '';
+                  my $selector = '';
+                  my $remote_ip = '';
+                  foreach my $signature ( @$arc_signatures ) {
+                    next if $signature->instance() ne $instance;
+                    $domain = $signature->domain();
+                    $selector = $signature->selector();
+                  }
+                  my $aar = $arc_auth_results->{$instance};
+                  $remote_ip = eval{ $aar->search({ 'isa' => 'entry', 'key' => 'iprev' })->children()->[0]->search({ 'isa' => 'subentry', 'key' => 'smtp.remote-ip'})->children()->[0]->value(); };
+                  $self->handle_exception( $@ );
+                  $remote_ip //= eval{ $aar->search({ 'isa' => 'entry', 'key' => 'iprev' })->children()->[0]->search({ 'isa' => 'subentry', 'key' => 'policy.iprev'})->children()->[0]->value(); };
+                  $self->handle_exception( $@ );
+
+                  $comment .= ' as['.$instance.'].d='.$domain.' as['.$instance.'].s='.$selector.' remote-ip['.$instance.']='.$remote_ip;
+                }
+              }
+            }
+            $self->dbgout( 'DMARCReject', "Policy overridden using ARC Chain: $comment", LOG_INFO );
+            $dmarc_result->reason( 'type' => 'local_policy', 'comment' => $comment );
         }
         elsif ( $is_whitelisted ) {
             $self->dbgout( 'DMARCReject', "Policy reject overridden by whitelist", LOG_INFO );
@@ -928,7 +976,7 @@ Mail::Milter::Authentication::Handler::DMARC
 
 =head1 VERSION
 
-version 2.20200107
+version 2.20200202
 
 =head1 DESCRIPTION
 
