@@ -1,7 +1,7 @@
 package Mail::Milter::Authentication::Metric;
 use strict;
 use warnings;
-our $VERSION = '2.20200107'; # VERSION
+our $VERSION = '2.20200202'; # VERSION
 
 
 use English qw{ -no_match_vars };
@@ -37,6 +37,7 @@ sub new {
         $self->{'prom'}->declare( 'authmilter_uptime_seconds_total', help => 'Number of seconds since server startup', type => 'counter' );
         $self->{'prom'}->declare( 'authmilter_processes_waiting', help => 'The number of authentication milter processes in a waiting state', type => 'gauge' );
         $self->{'prom'}->declare( 'authmilter_processes_processing', help => 'The number of authentication milter processes currently processing data', type => 'gauge' );
+        $self->{'prom'}->declare( 'authmilter_version', help => 'Running versions', type => 'gauge' );
     }
 
     bless $self, $class;
@@ -44,6 +45,18 @@ sub new {
 }
 
 
+sub set_versions {
+    my ( $self, $server ) = @_;
+    $self->{'prom'}->set( 'authmilter_version', 1, { version => $Mail::Milter::Authentication::VERSION, module => 'core', ident => $self->clean_label( $Mail::Milter::Authentication::Config::IDENT ) });
+    foreach my $Handler ( sort keys %{ $server->{ 'handler' } } ) {
+        next if $Handler eq '_Handler';
+        $self->{'prom'}->set( 'authmilter_version', 1, { version => $server->{ 'handler' }->{ $Handler }->get_version(), module => $Handler, ident => $self->clean_label( $Mail::Milter::Authentication::Config::IDENT ) });
+    }
+    return;
+}
+
+
+
 sub get_timeout {
     my ( $self ) = @_;
     my $config = get_config();
@@ -91,6 +104,35 @@ sub count {
 }
 
 
+sub set {
+    my ( $self, $args ) = @_;
+    return if ( ! $self->{ 'enabled' } );
+
+    my $gauge_id = $args->{ 'gauge_id' };
+    my $labels   = $args->{ 'labels' };
+    my $server   = $args->{ 'server' };
+    my $value    = $args->{ 'value' };
+
+    die 'metric set must define value' if ! defined $value;
+
+    $gauge_id =  $self->clean_label( $gauge_id );
+
+    my $clean_labels = {};
+    if ( $labels ) {
+        foreach my $l ( sort keys %$labels ) {
+            $clean_labels->{ $self->clean_label( $l ) } = $self->clean_label( $labels->{$l} );
+        }
+    }
+
+    $clean_labels->{ident} = $self->clean_label( $Mail::Milter::Authentication::Config::IDENT );
+
+    eval{ $self->{prom}->set( 'authmilter_' . $gauge_id, $value, $clean_labels ); };
+    ## TODO catch and re-throw timeouts
+
+    return;
+}
+
+
 sub send { ## no critic
     my ( $self, $server ) = @_;
     return;
@@ -102,8 +144,17 @@ sub register_metrics {
     return if ( ! $self->{ 'enabled' } );
 
     foreach my $metric ( keys %$hash ) {
-        my $help = $hash->{ $metric };
-        $self->{prom}->declare( 'authmilter_' . $metric, help => $help, type => 'counter');
+        my $data = $hash->{ $metric };
+        my $help;
+        my $type = 'counter';
+        if ( ref $data eq 'HASH' ) {
+            $help = $data->{help};
+            $type = $data->{type};
+        }
+        else {
+            $help = $data;
+        }
+        $self->{prom}->declare( 'authmilter_' . $metric, help => $help, type => $type);
         $self->{prom}->set( 'authmilter_' . $metric,0, { ident => $self->clean_label( $Mail::Milter::Authentication::Config::IDENT ) });
     }
     return;
@@ -167,6 +218,7 @@ sub child_handler {
         }
 
         if ( $request_uri eq '/metrics' ) {
+            $server->{'handler'}->{'_Handler'}->top_metrics_callback();
             $self->{prom}->set( 'authmilter_uptime_seconds_total', time - $self->{'start_time'}, { ident => $self->clean_label( $Mail::Milter::Authentication::Config::IDENT ) });
 
             print $socket "HTTP/1.0 200 OK\n";
@@ -300,7 +352,7 @@ Mail::Milter::Authentication::Metric
 
 =head1 VERSION
 
-version 2.20200107
+version 2.20200202
 
 =head1 DESCRIPTION
 
@@ -317,6 +369,10 @@ This object is used to store, modify, and report metrics.
 
 =head1 METHODS
 
+=head2 I<set_versions( $server )>
+
+Setup version metrics
+
 =head2 I<get_timeout()>
 
 Returns the current value of timeout for metrics operations.
@@ -339,6 +395,20 @@ $server is the current handler object
 
  count - number to increment by (defaults to 1)
 
+=head2 I<set($args)>
+
+Set the metric for the given counter
+Called from the base handler, do not call directly.
+$server is the current handler object
+
+ count_id - the name of the metric to act on
+
+ labels - hashref of labels to apply
+
+ server - the current server object
+
+ count - number to increment by (defaults to 1)
+
 =head2 I<send( $server )>
 
 Send metrics to the parent server process.
@@ -354,7 +424,7 @@ Expects a hashref of metric description, keyed on metric name.
 
 Called in the master process to periodically update some metrics
 
-=head2 I<child_handler( $server)>
+=head2 I<child_handler( $server )>
 
 Handle a metrics or http request in the child process.
 
